#!C:\Programme\Python25\python.exe
# -*- coding: UTF-8 -*-
import os, sys, subprocess, signal, datetime
from os.path import *

# ***** include the biocase.lib directory in the python sys path for importing *****
execfile( os.path.abspath( os.path.join( os.path.dirname( __file__ ), os.pardir, os.pardir, 'lib', 'biocase', 'adjustpath.py' ) ))
execfile( os.path.abspath( os.path.join( os.path.dirname( __file__ ), os.pardir, os.pardir, 'lib', 'biocase', 'appinit.py' ) ))

from biocase.configtool.general import *
from biocase.datasources import Datasource

archivefilename = form.getvalue('archivefilename')
dsa = form.getvalue('dsa')
ticket = form.getvalue('ticket')
filesize = form.getvalue('filesize')
errors = form.getvalue('errors')
action = form.getvalue('action')
transformer = form.getvalue('transformer')
schema = form.getvalue('schema')
schemaNs = form.getvalue('schemafile')

# check authentication!
authorize(form, dsa=dsa)
if dsa is None:
    execfile( os.path.abspath( os.path.join( os.path.dirname( __file__ ), 'main.cgi' ) ))
    sys.exit()

cfg = biocase.configuration.Cfg()
dsaObj = Datasource(name=dsa)
wrapperurl = dsaObj.getBioCASeAccessPoint()
archivePath = join(cfg.archiveWorkLocator, dsa)
archiveBin = join(cfg.archiveLibLocator, "archive.py")
proc_dwca = join(archivePath, dsa + '_DwCA_1.0.proc')
proc_xml = join(archivePath, archivefilename + ".proc")
log_dwca = join(archivePath, dsa + '_DwCA_1.0.log')
log_xml = join(archivePath, archivefilename + '.log')
dldPath = join(cfg.archiveDownloadLocator, dsa)

print('Content-Type: text/html\n')

def log_msg(file, text, tag = 'INFO'):
    ts = datetime.datetime.now()
    file.write('%-5s %02i-%02i %02i:%02i:%02i,%03i - %s\n' %(tag, ts.day, ts.month, ts.hour, ts.minute, ts. second, ts.microsecond/1000, text))

def create(xml, dwca):
    try:
        if xml:
            # Write config/query file
            q = open(join(archivePath, archivefilename + ".query.xml"), "w")
            q.write("<?xml version='1.0' encoding='UTF-8'?>\n")
            q.write("<request xmlns='http://www.biocase.org/schemas/protocol/1.3'>\n")
            q.write("  <header><type>search</type></header>\n")
            q.write("  <search>\n")
            q.write("    <requestFormat>"+schemaNs+"</requestFormat>\n")
            q.write("    <responseFormat start='0' limit='"+filesize+"'>"+schemaNs+"</responseFormat>\n")
            q.write("    <count>false</count>\n")
            q.write("  </search>\n")
            q.write("</request>\n")
            q.close()

            c = open(join(archivePath, archivefilename + ".config.xml"), "w")
            c.write("<?xml version='1.0' encoding='UTF-8'?>\n")
            c.write("<config>\n")
            c.write("<dsa>" + dsa + "</dsa>\n")
            c.write("<archivefilename>" + archivefilename + "</archivefilename>\n")
            c.write("<downloadfolder>" + cfg.archiveDownloadLocator + "</downloadfolder>\n")
            c.write("<ticket>" + ticket + "</ticket>\n")
            c.write("<wrapperurl>" + wrapperurl + "</wrapperurl>\n")
            c.write("<schema>"+schema+"</schema>\n")
            c.write("<schemans>"+schemaNs+"</schemans>\n")
            c.write("<consecutiveerrors>"+errors+"</consecutiveerrors>")
            c.write("</config>\n")
            c.close()

            # Try to open (an empty) logfile
            l = open(log_xml, 'w')
            l.close()

        if dwca:
            # Try to open (an empty) logfile
            l = open(log_dwca, 'w')
            l.close()

        # We need to set stdin and stdout because of a bug in Python:
        # Leaving these as None will result in a "handle not valid" error on IIS/Windows
        proc = subprocess.Popen([sys.executable, archiveBin, str(xml), str(dwca), dsa, archivefilename, archivePath, dldPath,
                          cfg.server.java, cfg.archiveLibLocator, cfg.kettleLibLocator, cfg.archiveWorkLocator, cfg.server.java_mem, cfg.server.sort_size,
                          str(transformer)], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        for err in proc.stderr:
            print err,

    except Exception, err:
        print err

def xml_cancel():	
    if exists(proc_xml):
        p = open(proc_xml, "w")
        p.write("cancel")
        p.close()
        print "Cancellation of "+archivefilename+" requested. Waiting for the running statement to finish..."
    else:
        print(archivefilename + " is not running.")

def dwca_cancel():
    if exists(proc_dwca):
        # Read process id
        q = open(proc_dwca, 'r')
        for line in q:
            id = line
        q.close()

        # First, try the Linux version
        try:
            try:
                os.kill(int(id), signal.SIGTERM)
                print "Process terminated by sending TERMINATE event."
            except AttributeError:
                # Wrong OS; raising error to get to get to Windows section
                raise
            except:
               print "Sorry, couldn't terminate process. Please wait for it to finish or cancel it manually using ps -kill (look for java)."

        # If that fails, try Windows taskkill with option f (forcefull)
        except:
            res = subprocess.call("taskkill /f /pid %s" % id)
            if res == 0:
                print "Process terminated using Windows Taskkill."
            else:
                print "Sorry, couldn't terminate process. Please wait for it to finish or cancel it manually using the task manager (look for java.exe)."

    else:
        print "Process already finished."            

# Main
if action == "xml_create":
    create(True, False)
elif action == "dwca_create":
    create(False, True)
elif action == 'both_create':
    create(True, True)
elif action == "xml_cancel":
    xml_cancel()
elif action == "dwca_cancel":
    dwca_cancel()
